/*----------------------------------------------------------------------
 * $Id: config_view.cpp,v 1.5 2005/10/03 03:13:30 ganzhorn Exp $             
 *
 *
 * Copyright (C)   2005            Salem Ganzhorn <eyekode@yahoo.com>
 *
 * This file is part of kccmp - Kernel Config CoMPare
 *                                                                      
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public License
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *----------------------------------------------------------------------*/
#include "config_view.hpp"

#ifdef KCCMP_QT_4
#include <QHBoxLayout>
#include <QTabWidget>
#include <QTableWidget>
#include <QListWidget>
#include <QHeaderView>
#include <QFileInfo>
#include <QDir>
#include <QLabel>
#else
#include <qtabwidget.h>
#include <qlayout.h>
#include <qfileinfo.h>
#include <qdir.h>
#include <qtable.h>
#include <qlabel.h>
#endif

#include <sstream>

using namespace std;

config_view::~config_view() {}

config_view::config_view(
    QWidget *parent
    )
    : QFrame( parent )
{
}

#ifdef KCCMP_QT_4
void
config_view::compare(
    const string& filename1,
    const string& filename2
    )
{
    {
	const QObjectList &trash = children();
	QObjectList::const_iterator i = trash.begin();
	while( trash.end() != i ) {
	    QObjectList::const_iterator j = i;
	    ++i;
	    delete *j;
	}
    }
    
    config c1;
    config c2;
    c1.read(filename1.c_str());
    c2.read(filename2.c_str());
    set<string> cunion,diff,in1,in2;
    config::analyze(c1,c2,cunion,diff,in1,in2);
    
    QTabWidget *tab = new QTabWidget(this);
    QLayout *l = new QHBoxLayout(this);
    l->addWidget(tab);

    QFileInfo fi1(filename1.c_str());
    QFileInfo fi2(filename2.c_str());
    QDir d1 = fi1.absoluteDir();
    QDir d2 = fi2.absoluteDir();
    QString id1 = d1.dirName();
    QString id2 = d2.dirName();
    if( id1 == id2 ) {
	id1 = fi1.fileName();
	id2 = fi2.fileName();
	if( id1 == id2 ) {
	    id1 = "first file";
	    id2 = "second file";
	}
    }
    add_diff_tab(tab,diff,c1,c2,id1,id2);
    add_set_tab(tab,in1,c1,id1);
    add_set_tab(tab,in2,c2,id2);
}

void
config_view::add_diff_tab(
    QTabWidget *tab,
    const set<string> &diff,
    const config      &c1,
    const config      &c2,
    const QString     &dir1,
    const QString     &dir2
    )
{
    QWidget *add_to_tab = NULL;
    if( diff.size() ) {
	QTableWidget *view = new QTableWidget();
	view->setColumnCount(3);
	view->setRowCount(diff.size());
	view->verticalHeader()->hide();
	QStringList labels;
	labels << "Config Var" << dir1 << dir2;
	view->setHorizontalHeaderLabels(labels);
	stringstream ss;
	view->setAlternatingRowColors(true);
	QTableWidgetItem *p = new QTableWidgetItem();
	p->setFlags(Qt::ItemIsEnabled);
	set<string>::const_iterator i = diff.begin();
	int row = 0;
	while( diff.end() != i ) {
	    QTableWidgetItem *item = p->clone();
	    item->setText(i->c_str());
	    view->setItem(row,0,item);
	    ss << *c1.find(*i);
	    item = p->clone();
	    item->setText(ss.str().c_str());
	    view->setItem(row,1,item);
	    ss.str("");
	    ss << *c2.find(*i);
	    item = p->clone();
	    item->setText(ss.str().c_str());
	    view->setItem(row,2,item);
	    ss.str("");
	    ++row;
	    ++i;
	}
	view->resizeColumnToContents(0);
	add_to_tab = view;
    } else {
	QLabel *l = new QLabel("N/A");
	add_to_tab = l;
    }
    tab->addTab(add_to_tab,"Different Values");
}

void
config_view::add_set_tab(
    QTabWidget *tab,
    const set<string> &info,
    const config &c,
    const QString &id
    )
{
    QWidget *add_to_tab=0;

    if( info.size() ) {
	QTableWidget *view = new QTableWidget();
	view->setColumnCount(2);
	view->setRowCount(info.size());
	view->verticalHeader()->hide();
	QStringList labels;
	labels << "Config Var" << "Value";
	view->setHorizontalHeaderLabels(labels);
	stringstream ss;
	view->setAlternatingRowColors(true);
	QTableWidgetItem *p = new QTableWidgetItem();
	p->setFlags(Qt::ItemIsEnabled);
	set<string>::const_iterator i = info.begin();
	int row = 0;
	while( info.end() != i ) {
	    QTableWidgetItem *item = p->clone();
	    item->setText(i->c_str());
	    view->setItem(row,0,item);
	    ss << *c.find(*i);
	    item = p->clone();
	    item->setText(ss.str().c_str());
	    view->setItem(row,1,item);
	    ss.str("");
	    ++row;
	    ++i;
	}
	view->resizeColumnToContents(0);
	add_to_tab = view;
    } else {
	QLabel *l = new QLabel("N/A");
	add_to_tab = l;
    }
    tab->addTab(add_to_tab,"Only in "+id);
}

#else
class ac_item : public QTableItem {
public:
    ac_item( QTable *t, EditType et, const QString &txt="" )
	: QTableItem( t, et, txt ) {}
    void paint( QPainter *p,
		const QColorGroup &cg,
		const QRect &cr,
		bool selected ) {
	QColorGroup g(cg);
	if( row()%2 ) {
	    g.setColor( QColorGroup::Base,cg.midlight() );
	}
	QTableItem::paint(p,g,cr,selected);
    }
};

void
config_view::compare(
    const string& filename1,
    const string& filename2
    )
{
    config c1;
    config c2;
    c1.read(filename1.c_str());
    c2.read(filename2.c_str());
    set<string> cunion,diff,in1,in2;
    config::analyze(c1,c2,cunion,diff,in1,in2);
    new QLabel(this,"WTF!!!");
    QTabWidget *tab = new QTabWidget(this);
    QBoxLayout *l = new QHBoxLayout(this);
    l->addWidget(tab);

    QFileInfo fi1(filename1.c_str());
    QFileInfo fi2(filename2.c_str());
    QDir d1 = fi1.dirPath(true);
    QDir d2 = fi2.dirPath(true);
    QString id1 = d1.dirName();
    QString id2 = d2.dirName();
    if( id1 == id2 ) {
	id1 = fi1.fileName();
	id2 = fi2.fileName();
	if( id1 == id2 ) {
	    id1 = "first file";
	    id2 = "second file";
	}
    }
    add_diff_tab(tab,diff,c1,c2,id1,id2);
    add_set_tab(tab,in1,c1,id1);
    add_set_tab(tab,in2,c2,id2);
    tab->show();
}

void
config_view::add_diff_tab(
    QTabWidget *tab,
    const set<string> &diff,
    const config      &c1,
    const config      &c2,
    const QString     &dir1,
    const QString     &dir2
    )
{
    QWidget *add_to_tab = NULL;
    if( diff.size() ) {
	QTable *view = new QTable();
	view->setNumCols(3);
	view->setNumRows(diff.size());
	view->verticalHeader()->hide();
	view->setLeftMargin(0);
	QStringList labels;
	labels << "Config Var" << dir1 << dir2;
	view->setColumnLabels(labels);
	stringstream ss;
	set<string>::const_iterator i = diff.begin();
	int row = 0;
	while( diff.end() != i ) {
	    QTableItem *item = new ac_item(view,QTableItem::Never);
	    item->setText(i->c_str());
	    view->setItem(row,0,item);
	    ss << *c1.find(*i);
	    item = new ac_item(view,QTableItem::Never);
	    item->setText(ss.str().c_str());
	    view->setItem(row,1,item);
	    ss.str("");
	    ss << *c2.find(*i);
	    item = new ac_item(view,QTableItem::Never);
	    item->setText(ss.str().c_str());
	    view->setItem(row,2,item);
	    ss.str("");
	    ++row;
	    ++i;
	}
	view->adjustColumn(0);
	add_to_tab = view;
    } else {
	QLabel *l = new QLabel("N/A",tab);
	add_to_tab = l;
    }
    tab->addTab(add_to_tab,"Different Values");
}

void
config_view::add_set_tab(
    QTabWidget *tab,
    const set<string> &info,
    const config &c,
    const QString &id
    )
{
    QWidget *add_to_tab=0;

    if( info.size() ) {
	QTable *view = new QTable();
	view->setNumCols(2);
	view->setNumRows(info.size());
	view->verticalHeader()->hide();
	view->setLeftMargin(0);
	QStringList labels;
	labels << "Config Var" << "Value";
	view->setColumnLabels(labels);
	stringstream ss;
	set<string>::const_iterator i = info.begin();
	int row = 0;
	while( info.end() != i ) {
	    QTableItem *item = new ac_item(view,QTableItem::Never);
	    item->setText(i->c_str());
	    view->setItem(row,0,item);
	    ss << *c.find(*i);
	    item = new ac_item(view,QTableItem::Never);
	    item->setText(ss.str().c_str());
	    view->setItem(row,1,item);
	    ss.str("");
	    ++row;
	    ++i;
	}
	view->adjustColumn(0);
	add_to_tab = view;
    } else {
	QLabel *l = new QLabel("N/A",tab);
	add_to_tab = l;
    }
    tab->addTab(add_to_tab,"Only in "+id);
}

#endif
